package com.ubuntu.builder;

import hudson.FilePath;
import hudson.Launcher;
import hudson.Extension;
import hudson.util.FormValidation;
import hudson.model.AbstractBuild;
import hudson.model.BuildListener;
import hudson.model.AbstractProject;
import hudson.remoting.*;
import hudson.tasks.Builder;
import hudson.tasks.BuildStepDescriptor;
import net.sf.json.JSONObject;
import org.kohsuke.stapler.DataBoundConstructor;
import org.kohsuke.stapler.StaplerRequest;
import org.kohsuke.stapler.QueryParameter;

import javax.servlet.ServletException;
import java.io.File;
import java.io.IOException;

/**
 * Sample {@link Builder}.
 *
 * <p>
 * When the user configures the project and enables this builder,
 * {@link DescriptorImpl#newInstance(StaplerRequest)} is invoked
 * and a new {@link HelloWorldBuilder} is created. The created
 * instance is persisted to the project configuration XML by using
 * XStream, so this allows you to use instance fields (like {@link #name})
 * to remember the configuration.
 *
 * <p>
 * When a build is performed, the {@link #perform(AbstractBuild, Launcher, BuildListener)}
 * method will be invoked. 
 *
 * @author Kohsuke Kawaguchi
 */
public class HelloWorldBuilder extends Builder {

  private static final int EXIT_SUCCESS = 0;
  private static final String CHECKOUT_DIRECTORY = new String("pbuilder_setup");
  
  private final String name;
  private final String pbuilderSetupBranch;
  private final String mainBranch;
  private final String packagingBranch;
  private final String workDirectory;
  private final String resultsDirectory;
  // Fields in config.jelly must match the parameter names in the "DataBoundConstructor"
  @DataBoundConstructor
  public HelloWorldBuilder(String pbuilderSetupBranch,
                           String mainBranch,
                           String packagingBranch,
                           String workDirectory,
                           String resultsDirectory) {
    this.pbuilderSetupBranch = pbuilderSetupBranch;
    this.mainBranch = mainBranch;
    this.packagingBranch = packagingBranch;
    this.workDirectory = workDirectory;
    this.resultsDirectory = resultsDirectory;
    this.name = null;
  }

  /**
   * We'll use this from the <tt>config.jelly</tt>.
   */
  public String getName() {
    return name;
  }

  public String getPbuilderSetupBranch() {
    return pbuilderSetupBranch;
  }

  public String getMainBranch() {
    return mainBranch;
  }

  public String getPackagingBranch() {
    return packagingBranch;
  }

  public String getWorkDirectory() {
    return workDirectory;
  }

  public String getResultsDirectory() {
    return resultsDirectory;
  }

  @Override
  public boolean perform(AbstractBuild build,
                         Launcher launcher,
                         BuildListener listener) {
    listener.getLogger().println("PBuilder config branch: " + pbuilderSetupBranch);    
    listener.getLogger().println("Main branch: " + mainBranch);
    listener.getLogger().println("Packaging branch: " + packagingBranch);
    listener.getLogger().println("Work directory: " + workDirectory);
    listener.getLogger().println("Results directory: " + resultsDirectory);
    
    try {
      FilePath workDir = build.getWorkspace().child(workDirectory);
      workDir.mkdirs();
      
      FilePath resultDir = build.getWorkspace().child(resultsDirectory);
      resultDir.mkdirs();

      FilePath checkoutDir = build.getWorkspace().child(this.CHECKOUT_DIRECTORY);
      // checkoutDir.deleteRecursive();

      if(!getOrBranchPBuilderSetup(build, launcher, listener)) {
        listener.getLogger().println("Problem branching or updating pbuilder setup.");
        return false;
      }
      // Get the pbuilder setup
      /*if(launcher.launch().
         cmds("bzr",
              "checkout",
              pbuilderSetupBranch,
              build.getWorkspace().child(this.CHECKOUT_DIRECTORY).toString()).
         stdout(listener.getLogger()).
         stderr(listener.getLogger()).
         pwd(build.getWorkspace()).
         join() != this.EXIT_SUCCESS) {
        
        listener.getLogger().println("Problem branching pbuilder setup ... aborting.");
        return false;
        }*/

      if(launcher.launch().
         cmds("sudo",
              "./Builder.sh",
              "-w",
              "work",
              "-h",
              "./hooks",
              "-r",
              resultDir.toString(),//"../results",
              "-m",
              mainBranch,
              "-p",
              packagingBranch).
         pwd(build.getWorkspace() + "/pbuilder_setup").
         stdout(listener.getLogger()).
         stderr(listener.getLogger()).
         join() != this.EXIT_SUCCESS) {

        listener.getLogger().println("Problem invoking pbuilder ... aborting.");
        return false;
      }
      
    } catch(Exception e) {
      listener.getLogger().println(e);
      return false;
    }
    return true;
  }

  private boolean getOrBranchPBuilderSetup(AbstractBuild build,
                                           Launcher launcher,
                                           BuildListener listener) throws IOException,
  InterruptedException {

    FilePath pbuilderSetupDir = build.getWorkspace().child(this.CHECKOUT_DIRECTORY);

    int exitCode = -1;
    
    if(pbuilderSetupDir.exists()) {
      exitCode = launcher.launch().
          cmds("bzr",
               "pull",
               pbuilderSetupBranch).          
          stdout(listener.getLogger()).
          stderr(listener.getLogger()).
          pwd(pbuilderSetupDir).
          join();
    } else {
      exitCode = launcher.launch().
          cmds("bzr",
               "checkout",
               pbuilderSetupBranch,
               pbuilderSetupDir.toString()).
          stdout(listener.getLogger()).
          stderr(listener.getLogger()).
          pwd(build.getWorkspace()).
          join();
    }

    return exitCode == EXIT_SUCCESS;
  }
  
  // Overridden for better type safety.
  // If your plugin doesn't really define any property on Descriptor,
  // you don't have to do this.
  @Override
  public DescriptorImpl getDescriptor() {
    return (DescriptorImpl)super.getDescriptor();
  }

  /**
   * Descriptor for {@link HelloWorldBuilder}. Used as a singleton.
   * The class is marked as public so that it can be accessed from views.
   *
   * <p>
   * See <tt>src/main/resources/hudson/plugins/hello_world/HelloWorldBuilder/*.jelly</tt>
   * for the actual HTML fragment for the configuration screen.
   */
  @Extension // This indicates to Jenkins that this is an implementation of an extension point.
  public static final class DescriptorImpl extends BuildStepDescriptor<Builder> {
    /**
     * To persist global configuration information,
     * simply store it in a field and call save().
     *
     * <p>
     * If you don't want fields to be persisted, use <tt>transient</tt>.
     */
    private boolean useFrench;

    /**
     * Performs on-the-fly validation of the form field 'name'.
     *
     * @param value
     *      This parameter receives the value that the user has typed.
     * @return
     *      Indicates the outcome of the validation. This is sent to the browser.
     */
    public FormValidation doCheckName(@QueryParameter String value)
        throws IOException, ServletException {
      if (value.length() == 0)
        return FormValidation.error("Please set a name");
      if (value.length() < 4)
        return FormValidation.warning("Isn't the name too short?");
      return FormValidation.ok();
    }

    /**
     * Performs on-the-fly validation of the form field 'Main branch'.
     *
     * @param value
     *      This parameter receives the value that the user has typed.
     * @return
     *      Indicates the outcome of the validation. This is sent to the browser.
     */
    public FormValidation doCheckPbuilderSetupBranch(@QueryParameter String value)
        throws Exception, IOException, ServletException {
      if(!value.startsWith("lp:"))
        return FormValidation.error("Only launchpad registered projects are supported at the present moment.");

      Launchpad.Branch branch = Launchpad.getBranch(value.replace("lp:", ""));
      if(branch == null)
        return FormValidation.error("No such branch could be found in Launchpad.");
      
      
      return FormValidation.ok();
    }

    /**
     * Performs on-the-fly validation of the form field 'Main branch'.
     *
     * @param value
     *      This parameter receives the value that the user has typed.
     * @return
     *      Indicates the outcome of the validation. This is sent to the browser.
     */
    public FormValidation doCheckMainBranch(@QueryParameter String value)
        throws Exception, IOException, ServletException {
      if(!value.startsWith("lp:"))
        return FormValidation.error("Only launchpad registered projects are supported at the present moment.");

      if(Launchpad.getBranch(value.replace("lp:", "")) == null)
        return FormValidation.error("No such branch could be found in Launchpad.");

      return FormValidation.ok();
    }

    /**
     * Performs on-the-fly validation of the form field 'Packaging branch'.
     *
     * @param value
     *      This parameter receives the value that the user has typed.
     * @return
     *      Indicates the outcome of the validation. This is sent to the browser.
     */
    public FormValidation doCheckPackagingBranch(@QueryParameter String value)
        throws Exception, IOException, ServletException {
      if(!value.startsWith("lp:"))
        return FormValidation.error("Only launchpad registered projects are supported at the present moment.");

      if(Launchpad.getBranch(value.replace("lp:", "")) == null)
        return FormValidation.error("No such branch could be found in Launchpad.");

      return FormValidation.ok();
    }

    public boolean isApplicable(Class<? extends AbstractProject> aClass) {
      // Indicates that this builder can be used with all kinds of project types 
      return true;
    }

    /**
     * This human readable name is used in the configuration screen.
     */
    public String getDisplayName() {
      return "Debian PBuilder";
    }

    @Override
        public boolean configure(StaplerRequest req, JSONObject formData) throws FormException {
      // To persist global configuration information,
      // set that to properties and call save().
      useFrench = formData.getBoolean("useFrench");
      // ^Can also use req.bindJSON(this, formData);
      //  (easier when there are many fields; need set* methods for this, like setUseFrench)
      save();
      return super.configure(req,formData);
    }

    /**
     * This method returns true if the global configuration says we should speak French.
     */
    public boolean useFrench() {
      return useFrench;
    }
  }
}

